package com.persagy.apm.dmpalarm.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.annotation.Resource;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.dmpalarm.criteria.CriteriaUtils;
import com.persagy.apm.dmpalarm.criteria.JsonCriteria;
import com.persagy.apm.dmpalarm.enumeration.EnumAlarmMessageType;
import com.persagy.apm.dmpalarm.model.AlarmItem;
import com.persagy.apm.dmpalarm.model.DmpMessage;
import com.persagy.apm.dmpalarm.service.IAlarmItemService;
import com.persagy.apm.dmpalarm.utils.DatePatternStyle;
import com.persagy.apm.dmpalarm.web.MapResponse;
import com.persagy.apm.dmpalarm.web.PagedResponse;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 报警条目(AlarmItem) service层
 *
 * @author lixing
 * @version V1.0 2021-12-09 10:44:19
 */
@Service("dmpAlarmItemService")
@Slf4j
public class AlarmItemServiceImpl extends BaseServiceImpl<BaseMapper<AlarmItem>, AlarmItem>
        implements IAlarmItemService {

    @Override
    public PagedResponse<AlarmItem> query(JsonCriteria criteria) {
        // 报警条目查询中的0代表的是通用
        try {
            addDefaultQueryCondition(criteria, "projectId", "0");
            addDefaultQueryCondition(criteria, "groupCode", "0");
            addDefaultQueryCondition(criteria, "appId", "0");
            QueryWrapper<AlarmItem> queryWrapper = criteriaUtils.handleAlarmConditions(criteria, AlarmItem.class);
            return criteriaUtils.getPagedResponse(criteria, AlarmItem.class, queryWrapper, getBaseMapper());
        } catch (Exception e) {
            log.error("AlarmItem query page error JsonCriteria is {}",criteria.toString(), e);
        }
        return new PagedResponse<>();
    }

    /**
     * @description: 添加默认查询条件
     * 例如：当传入的查询条件中projectId = 1时，实际需要查询projectId = [0, 1]的数据，使用此方法将0添加到查询条件中
     * @param: criteria
     * @param: paramName
     * @param: defaultCondition
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/5 5:29 下午
     * @version: V1.0
     */
    private void addDefaultQueryCondition(JsonCriteria criteria, String paramName, String defaultCondition) {
        // 这里不校验已有查询条件和默认查询条件是否重复
        JSONObject queryBody = criteria.getCriteria();
        Object obj = queryBody.get(paramName);
        if (obj == null) {
            queryBody.put(paramName, defaultCondition);
        } else if (obj instanceof JSONArray || obj instanceof List) {
            ((JSONArray) obj).add(defaultCondition);
        } else {
            List<String> conditions = new ArrayList<>();
            conditions.add(defaultCondition);
            conditions.add(obj.toString());
            queryBody.put(paramName, conditions);
        }
    }

    @Transactional
    @Override
    public MapResponse create(AlarmItem entity) {
        MapResponse response = new MapResponse();
        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }

        entity.setCreateUser(alarmParam.userId);
        entity.setCreateTime(new Date());
        save(entity);
        response.add("id", entity.getId());
        /* 拼装需要推送至消息队列的消息 */
        DmpMessage msg = new DmpMessage();
        msg.setMid(UUID.randomUUID().toString());
        msg.setType(EnumAlarmMessageType.ALARM_ITEM_CREATE.getValue());
        msg.setGroupCode(alarmParam.groupCode);
        msg.setProjectId(alarmParam.projectId);
        msg.setTargetId(entity.getId());
        msg.setStr1(entity.getCode());
        msg.setAppId(alarmParam.appId);
        msg.setSendTime(DateUtil.format(new Date(), DatePatternStyle.PATTERN_YYYYMMDDHHMMSS));
        response.add(msg);
        return response;
    }

    @Transactional
    @Override
    public MapResponse update(AlarmItem entity) {
        MapResponse response = new MapResponse();
        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        entity.setUpdateUser(alarmParam.userId);
        entity.setUpdateTime(new Date());
        saveOrUpdate(entity);
        response.add("id", entity.getId());
        /* 拼装需要推送至消息队列的消息 */
        DmpMessage msg = new DmpMessage();
        msg.setMid(UUID.randomUUID().toString());
        msg.setType(EnumAlarmMessageType.ALARM_ITEM_UPDATE.getValue());
        msg.setGroupCode(alarmParam.groupCode);
        msg.setProjectId(alarmParam.projectId);
        msg.setTargetId(entity.getId());
        msg.setStr1(entity.getCode());
        msg.setAppId(alarmParam.appId);
        msg.setSendTime(DateUtil.format(new Date(), DatePatternStyle.PATTERN_YYYYMMDDHHMMSS));
        response.add(msg);
        return response;
    }

    /**
     * @description: 根据itemCodes获取itemCode, alarmItem的映射map
     * @param: itemCodes 报警条目编码列表
     * @return: java.util.HashMap<java.lang.String, com.persagy.dmp.alarm.model.AlarmItem>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/20 2:25 下午
     * @version: V1.0
     */
    public Map<String, AlarmItem> getItemCodeModelMap(List<String> itemCodes) {
        HashMap<String, AlarmItem> itemCodeModelMap = new HashMap<>();
        if (CollectionUtils.isEmpty(itemCodes)) {
            return itemCodeModelMap;
        }
        JsonCriteria criteria = new JsonCriteria();
        criteria.getCriteria().put("itemCode", itemCodes);
        PagedResponse<AlarmItem> response = query(criteria);
        if (response == null) {
            return itemCodeModelMap;
        }
        List<AlarmItem> data = response.getData();
        if (CollectionUtils.isEmpty(data)) {
            return itemCodeModelMap;
        }
        return data.stream().collect(
                Collectors.toMap(AlarmItem::getCode, itemModel -> itemModel, (k1, k2) -> k1)
        );
    }

    public AlarmItem getAlarmItemByItemCode(String itemCode, String projectId, String groupCode, String appId) {
        JsonCriteria criteria = new JsonCriteria();
        criteria.getCriteria().put("code", itemCode);
        if (StringUtils.isNotEmpty(projectId)) {
            criteria.getCriteria().put("projectId", projectId);
        }
        if (StringUtils.isNotEmpty(groupCode)) {
            criteria.getCriteria().put("groupCode", groupCode);
        }
        if (StringUtils.isNotEmpty(appId)) {
            criteria.getCriteria().put("appId", appId);
        }
        List<AlarmItem> AlarmItems = queryList(criteria);
        if (CollectionUtils.isEmpty(AlarmItems)) {
            return null;
        }
        return AlarmItems.get(0);
    }
    @Override
    public List<AlarmItem> queryList(JsonCriteria criteria) {
        PagedResponse<AlarmItem> pagedResponse = query(criteria);
        if (pagedResponse == null) {
            return null;
        }
        List<AlarmItem> data = pagedResponse.getData();
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        return data;
    }
}
