package com.persagy.apm.dmpalarm.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.persagy.apm.dmpalarm.config.DmpParameterStorage;
import com.persagy.apm.dmpalarm.criteria.JsonCriteria;
import com.persagy.apm.dmpalarm.enumeration.EnumAlarmMessageType;
import com.persagy.apm.dmpalarm.model.AlarmItem;
import com.persagy.apm.dmpalarm.model.AlarmTarget;
import com.persagy.apm.dmpalarm.model.DmpMessage;
import com.persagy.apm.dmpalarm.service.IAlarmItemService;
import com.persagy.apm.dmpalarm.service.IAlarmTargetService;
import com.persagy.apm.dmpalarm.utils.DatePatternStyle;
import com.persagy.apm.dmpalarm.web.MapResponse;
import com.persagy.apm.dmpalarm.web.PagedResponse;
import com.persagy.common.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * 报警匹配条件(AlarmTarget) service层
 *
 * @author lixing
 * @version V1.0 2021-12-09 10:44:19
 */
@Service
@Slf4j
public class AlarmTargetServiceImpl extends BaseServiceImpl<BaseMapper<AlarmTarget>, AlarmTarget>
        implements IAlarmTargetService {
    @Resource(name="dmpAlarmItemService")
    private IAlarmItemService alarmItemService;

    public PagedResponse<AlarmTarget> query(JsonCriteria criteria) {
        try {
            // 默认查询未删除的数据
            criteria.getCriteria().put("valid", 1);
            QueryWrapper<AlarmTarget> queryWrapper = criteriaUtils.handleAlarmConditions(criteria, AlarmTarget.class);
            PagedResponse<AlarmTarget> resp = criteriaUtils.getPagedResponse(criteria, AlarmTarget.class, queryWrapper, getBaseMapper());
            PagedResponse<AlarmTarget> result = new PagedResponse<>();
            result.setCount(resp.getCount());
            List<AlarmTarget> dataList = resp.getData();
            if (dataList != null && dataList.size() > 0) {
                Map<String, AlarmItem> itemCodeModelMap = getItemCodeModelMap(dataList);
                List<AlarmTarget> collect = dataList.stream().map(entity -> {
                    // 获取报警条目的触发条件
                    String itemCode = entity.getItemCode();
                    entity.setCondition(getCondition(entity, itemCodeModelMap.get(itemCode)));
                    return entity;
                }).collect(Collectors.toList());
                result.setData(collect);
            }
            return result;
        } catch (Exception e) {
           log.error("alarmTarget query page error JsonCriteria is {}",criteria.toString(),e);
        }
        return new PagedResponse<>();
    }

    @Transactional
    public MapResponse create(AlarmTarget entity) {
        MapResponse response = new MapResponse();

        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        if (org.springframework.util.StringUtils.isEmpty(entity.getItemCode())) {
            response.setFail("报警条目编码必填");
        }
        if (org.springframework.util.StringUtils.isEmpty(entity.getClassCode())) {
            response.setFail("对象类型编码必填");
        }

        // 获取设备条目对象
        AlarmItem alarmItem = alarmItemService.getAlarmItemByItemCode(entity.getItemCode(), alarmParam.projectId, alarmParam.groupCode, alarmParam.appId);
        if (alarmItem == null) {
            response.setFail("无法获取到对应的报警条目，创建失败！");
            return response;
        }

        entity.setId(UUID.randomUUID().toString());
        entity.setCreateUser(alarmParam.userId);
        entity.setCreateTime(new Date());
        entity.setProjectId(alarmParam.projectId);
        entity.setGroupCode(alarmParam.groupCode);
        // 继承alarmItem的一些属性
        if (org.springframework.util.StringUtils.isEmpty(entity.getLevel())) {
            entity.setLevel(alarmItem.getLevel());
        }
        save(entity);
        response.add("id", entity.getId());
        /* 拼装需要推送至消息队列的消息 */
        DmpMessage msg = new DmpMessage();
        msg.setMid(UUID.randomUUID().toString());
        msg.setType(EnumAlarmMessageType.ALARM_TARGET_CREATE.getValue());
        msg.setGroupCode(alarmParam.groupCode);
        msg.setProjectId(alarmParam.projectId);
        msg.setTargetId(entity.getId());
        msg.setStr1(entity.getItemCode());
        msg.setStr2(entity.getClassCode());
        msg.setAppId(alarmParam.appId);
        msg.setSendTime(DateUtil.format(new Date(), DatePatternStyle.PATTERN_YYYYMMDDHHMMSS));
        response.add(msg);
        return response;
    }


    /**
     * @description: 判断匹配条件更新是否会影响报警定义
     * @param: param 匹配条件更新信息
     * @param: alarmTarget 匹配条件字段
     * @return: boolean
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 8:55 下午
     * @version: V1.0
     */
    private boolean influenceConfigs(AlarmTarget param, AlarmTarget alarmTarget) {
//        boolean itemCodeChange = !StringUtils.isEmpty(param.getItemCode()) &&
//                !param.getItemCode().equals(alarmTarget.getItemCode());
        boolean levelChange = !org.springframework.util.StringUtils.isEmpty(param.getLevel()) &&
                !param.getLevel().equals(alarmTarget.getLevel());
        boolean classCodeChange = !org.springframework.util.StringUtils.isEmpty(param.getClassCode()) &&
                !param.getClassCode().equals(alarmTarget.getClassCode());
        boolean concernChange = param.getConcern() != null &&
                !param.getConcern().equals(alarmTarget.getConcern());
        boolean openChange = param.getOpen() != null &&
                !param.getOpen().equals(alarmTarget.getOpen());
        boolean conditionChange = !StringUtils.isEmpty(param.getCondition()) &&
                !param.getCondition().equals(alarmTarget.getCondition());

        return levelChange || classCodeChange || concernChange || openChange || conditionChange;
    }

    @Transactional
    public MapResponse update(AlarmTarget param) {
        MapResponse response = new MapResponse();
        AlarmTarget alarmTarget = getById(param.getId());
        if (alarmTarget == null) {
            response.setFail("无法获取到要更新的数据！");
            return response;
        }
        AlarmParam alarmParam = prepareParam(response);
        if (alarmParam == null) {
            return response;
        }
        /* 判断是否会影响报警定义 */
        boolean influenceConfigs = influenceConfigs(param, alarmTarget);

        if (param.getItemCode() != null && !param.getItemCode().isEmpty()) {
            alarmTarget.setItemCode(param.getItemCode());
        }
//        if (param.getClassCode() != null && !param.getClassCode().isEmpty()) {
//            alarmTarget.setClassCode(param.getClassCode());
//        }
        if (param.getLevel() != null && !param.getLevel().isEmpty()) {
            alarmTarget.setLevel(param.getLevel());
        }
        if (param.getConcern() != null) {
            alarmTarget.setConcern(param.getConcern());
        }
        if (param.getOpen() != null) {
            alarmTarget.setOpen(param.getOpen());
        }
        if (param.getCondition() != null && !param.getCondition().isEmpty()) {
            alarmTarget.setCondition(param.getCondition());
        }
        if (param.getValid() != null) {
            alarmTarget.setValid(param.getValid());
        }
        alarmTarget.setUpdateUser(alarmParam.userId);
        alarmTarget.setUpdateTime(new Date());
        saveOrUpdate(alarmTarget);
        response.add("id", alarmTarget.getId());
        if (influenceConfigs) {
            /* 拼装需要推送至消息队列的消息 */
            DmpMessage msg = new DmpMessage();
            msg.setMid(UUID.randomUUID().toString());
            msg.setType(EnumAlarmMessageType.ALARM_TARGET_UPDATE.getValue());
            msg.setGroupCode(alarmParam.groupCode);
            msg.setProjectId(alarmParam.projectId);
            msg.setTargetId(alarmTarget.getId());
            msg.setStr1(alarmTarget.getItemCode());
            msg.setStr2(alarmTarget.getClassCode());
            msg.setAppId(alarmParam.appId);
            msg.setSendTime(DateUtil.format(new Date(), DatePatternStyle.PATTERN_YYYYMMDDHHMMSS));
            response.add(msg);
        }
        return response;
    }

    @Transactional
    public MapResponse delete(AlarmTarget param) {
        MapResponse response = new MapResponse();
        param.setValid(0);
        update(param);

        /* 拼装需要推送至消息队列的消息 */
        DmpMessage msg = new DmpMessage();
        msg.setMid(UUID.randomUUID().toString());
        msg.setType(EnumAlarmMessageType.ALARM_TARGET_DELETE.getValue());
        msg.setGroupCode(DmpParameterStorage.getGroupCode());
        msg.setProjectId(DmpParameterStorage.getProjectId());
        msg.setTargetId(param.getId());
        msg.setStr1(param.getItemCode());
        msg.setStr2(param.getClassCode());
        msg.setAppId(DmpParameterStorage.getAppId());
        msg.setSendTime(DateUtil.format(new Date(), DatePatternStyle.PATTERN_YYYYMMDDHHMMSS));
        response.add(msg);
        return response;
    }

    /**
     * @description: 根据给定的一组target，获取target所属的报警条目itemCode, alarmItem的映射map
     * @param: targetModelList
     * @return: java.util.HashMap<java.lang.String, com.persagy.dmp.alarm.model.AlarmItem>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/20 2:32 下午
     * @version: V1.0
     */
    @Override
    public Map<String, AlarmItem> getItemCodeModelMapByModelList(List<AlarmTarget> targetModelList) {
        List<String> itemCodeList = targetModelList.stream().map(AlarmTarget::getItemCode).collect(Collectors.toList());
        return alarmItemService.getItemCodeModelMap(itemCodeList);
    }

    /**
     * @description: 根据给定的一组target，获取target所属的报警条目itemCode, alarmItem的映射map
     * @param: targetList
     * @return: java.util.HashMap<java.lang.String, com.persagy.dmp.alarm.model.AlarmItem>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/20 2:45 下午
     * @version: V1.0
     */
    public Map<String, AlarmItem> getItemCodeModelMap(List<AlarmTarget> targetList) {
        List<String> itemCodeList = targetList.stream().map(AlarmTarget::getItemCode).collect(Collectors.toList());
        return alarmItemService.getItemCodeModelMap(itemCodeList);
    }

    /**
     * @description: 获取匹配条件的触发条件：匹配条件自己的触发条件与所属报警条目的触发条件取并集
     * @param: AlarmTarget 匹配条件对象
     * @param: AlarmItem 报警条目对象
     * @return: ObjectNode
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/20 2:07 下午
     * @version: V1.0
     */
    @Override
    public JSONObject getCondition(AlarmTarget AlarmTarget, AlarmItem alarmItem) {
        JSONObject result = null;
        if (AlarmTarget != null && AlarmTarget.getCondition() != null) {
            result = AlarmTarget.getCondition();
        } else if (alarmItem != null) {
            result = alarmItem.getCondition();
        }
        return result;
    }

    /**
     * @description: 根据报警条目编码和设备类型编码获取匹配条件对象
     * @param: itemCode 报警条目编码
     * @param: classCode 设备类型编码
     * @param: projectId 项目id
     * @return: com.persagy.dmp.alarm.model.AlarmTarget
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 11:53 上午
     * @version: V1.0
     */
    public AlarmTarget getAlarmTarget(String itemCode, String classCode, String projectId) {
        JsonCriteria criteria = new JsonCriteria();
        criteria.getCriteria().put("itemCode", itemCode);
        criteria.getCriteria().put("classCode", classCode);
        criteria.getCriteria().put("projectId", projectId);
        PagedResponse<AlarmTarget> pagedResponse = query(criteria);
        if (pagedResponse == null) {
            return null;
        }
        List<AlarmTarget> data = pagedResponse.getData();
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        return data.get(0);
    }

    /**
     * @description: 根据报警条目编码获取匹配条件对象
     * @param: itemCode 报警条目编码
     * @param: projectId 项目id
     * @return: java.util.List<com.persagy.dmp.alarm.model.AlarmTarget>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 12:13 下午
     * @version: V1.0
     */
    @Override
    public List<AlarmTarget> getAlarmTargetsByItemCode(String itemCode, String projectId) {
        JsonCriteria criteria = new JsonCriteria();
        criteria.getCriteria().put("itemCode", itemCode);
        criteria.getCriteria().put("projectId", projectId);
        PagedResponse<AlarmTarget> pagedResponse = query(criteria);
        if (pagedResponse == null) {
            return null;
        }
        List<AlarmTarget> data = pagedResponse.getData();
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        return data;
    }

    @Override
    public AlarmTarget get(String targetId) {
        return getById(targetId);
    }
}
